/* Code for

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Appendix J of
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Semih Uslu: Pricing and Liquidity in Decentralized Asset Markets
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


Purpose of this file is to clean TRACE, using the code provided by Dick-Nielsen (2014).	*/

Libname TRACE '/folders/myfolders/sasuser.v94'; /* Name your own directory here. */
%let in = trace_enhanced_raw;
%let out = traceCLEAN;

/*	Erase the log and output.	*/
dm 'clear log'; dm 'clear output';

/*	Post 2012 change. Cleans the data reported after Feb. 6th 2012.	*/
data temp_raw
	 	temp_deleteI_NEW (keep = cusip_id entrd_vol_qt rptd_pr trd_exctn_dt trd_exctn_tm rpt_side_cd cntra_mp_id msg_seq_nb)
		temp_deleteII_NEW (keep = cusip_id entrd_vol_qt rptd_pr trd_exctn_dt trd_exctn_tm rpt_side_cd cntra_mp_id orig_msg_seq_nb);
	set trace.&IN;
		where trd_rpt_dt ge '06Feb2012'd;
		if cusip_id = '' then delete;
		if trc_st in ('X', 'C') then output temp_deleteI_NEW;
			else if trc_st in ('Y') then output temp_deleteII_NEW;
				else output temp_raw;
run;

proc sql;
	create table temp_raw2 as
	select * from temp_raw as a,
	( (select cusip_id, entrd_vol_qt, rptd_pr, trd_exctn_dt, trd_exctn_tm, rpt_side_cd, cntra_mp_id, msg_seq_nb from temp_raw)
		except
		(select cusip_id, entrd_vol_qt, rptd_pr, trd_exctn_dt, trd_exctn_tm, rpt_side_cd, cntra_mp_id, msg_seq_nb from temp_deleteI_NEW) ) as b
	where a.cusip_id = b.cusip_id
	and a.entrd_vol_qt = b.entrd_vol_qt
	and a.rptd_pr = b.rptd_pr
	and a.trd_exctn_dt = b.trd_exctn_dt
	and a.trd_exctn_tm = b.trd_exctn_tm
	and a.rpt_side_cd = b.rpt_side_cd
	and a.cntra_mp_id = b.cntra_mp_id
	and a.msg_seq_nb = b.msg_seq_nb;
quit;

proc sql;
	create table temp_raw3_NEW as
	select * from temp_raw2 as a,
	( (select cusip_id, entrd_vol_qt, rptd_pr, trd_exctn_dt, trd_exctn_tm, rpt_side_cd, cntra_mp_id, msg_seq_nb from temp_raw2)
		except
		(select cusip_id, entrd_vol_qt, rptd_pr, trd_exctn_dt, trd_exctn_tm, rpt_side_cd, cntra_mp_id, orig_msg_seq_nb from temp_deleteII_NEW) ) as b
	where a.cusip_id = b.cusip_id
	and a.entrd_vol_qt = b.entrd_vol_qt
	and a.rptd_pr = b.rptd_pr
	and a.trd_exctn_dt = b.trd_exctn_dt
	and a.trd_exctn_tm = b.trd_exctn_tm
	and a.rpt_side_cd = b.rpt_side_cd
	and a.cntra_mp_id = b.cntra_mp_id
	and a.msg_seq_nb = b.msg_seq_nb;
quit;

/*	Pre 2012 change. Cleans data reported before Feb. 6th 2012.	*/
data temp_raw temp_delete (keep = TRD_RPT_DT ORIG_MSG_SEQ_NB); set trace.&IN; where trd_rpt_dt < '06Feb2012'd;
	if cusip_id = '' then delete;
	if trc_st = 'C' then output temp_delete;
		else if trc_st = 'W' then output temp_delete temp_raw;
			else output temp_raw;
run;

proc sql;
	create table temp_raw2 as
	select * from temp_raw as a,
	( (select MSG_SEQ_NB, TRD_RPT_DT from temp_raw)
		except
		(select ORIG_MSG_SEQ_NB, TRD_RPT_DT from temp_delete) ) as b
		where a.msg_seq_nb = b.msg_seq_nb and a.TRD_RPT_DT = b.TRD_RPT_DT;
quit;

data reversal temp_raw3; set temp_raw2;
	N = _N_;
	if asof_cd = 'R' then output reversal;
		else output temp_raw3;
run;

proc sort data = reversal (drop = N) nodupkey; 
	by trd_exctn_dt cusip_id trd_exctn_tm rptd_pr entrd_vol_qt rpt_side_cd cntra_mp_id trd_rpt_dt trd_rpt_tm MSG_SEQ_NB; run;

proc sort data = temp_raw3; by trd_exctn_dt cusip_id trd_exctn_tm rptd_pr entrd_vol_qt rpt_side_cd cntra_mp_id; run;

data reversal2;
	merge temp_raw3 (in = qqq) reversal (in = qq);
		by trd_exctn_dt cusip_id trd_exctn_tm rptd_pr entrd_vol_qt rpt_side_cd cntra_mp_id;
			if qq = 1;
			if qqq = 1;
			if trd_exctn_dt < trd_rpt_dt;
run;
proc sort data = reversal2; by N; run;
proc sort data = temp_raw3; by N; run;

data temp_raw4; merge reversal2 (in = qq) temp_raw3; by N; if qq = 0; run;

/*	Combine PRE and POST files into one.	*/
data temp_raw_comb (drop = N asof_cd trd_rpt_dt trd_rpt_tm MSG_SEQ_NB trc_st orig_msg_seq_nb);
	set temp_raw4 temp_raw3_NEW;
run;

/*	Agency transaction filtering.	*/
data temp_raw6 (drop = agency); set temp_raw_comb;
	if rpt_side_cd = 'B' then agency = buy_cpcty_cd;
		else if rpt_side_cd = 'S' then agency = sell_cpcty_cd;
	if agency = 'A' and cntra_mp_id = 'C' and CMSN_TRD = 'N' then delete;
run;
proc sort data = temp_raw6; by cusip_id trd_exctn_dt; run;

data temp_raw7; set temp_raw6;
	if cntra_mp_id = 'D' and rpt_side_cd = 'B' then delete;				/*	Deletes one side of the interdealer trades.			*/
	if cntra_mp_id = 'D' and rpt_side_cd = 'S' then rpt_side_cd = 'D';	/*	Renames reporting party to indicate interdealer tra.*/
	if wis_fl = 'N';													/*	Deletes	WI trades.									*/
	if trdg_mkt_cd in ('SP2', 'P1', 'P2') then delete;					/*	Deletes non-secondary market trades.				*/
	if spcl_trd_fl = 'Y' then delete;									/*	Deletes if bond trades under special circumstances.	*/
	if scrty_type_cd = 'C' or scrty_type_cd = '';						/*	Deletes equity-linked bonds.						*/
	if (days_to_sttl_ct+0) < 6;											/*	Deletes if days to settlement is greater than 5.	*/
	if sale_cndtn_cd = 'C' then delete;									/*	Deletes if it is not a cash sale.					*/
	if cmsn_trd = 'N';													/*	Deletes commission trades.							*/
	if agu_qsr_id in ('A', 'Q') then delete;							/*	Deletes automatic give-up trades.					*/
run;

data a6_trace_enhanced (keep= complete_cusip trans_date trans_time flat_price yield trdsize issuer_cusip); 
	set temp_raw7; where cusip_id ne '';
	issuer_cusip = substr(cusip_id, 1, 6);
	rename	cusip_id = complete_cusip
			trd_exctn_dt = trans_date
			trd_exctn_tm = trans_time
			rptd_pr = flat_price
			entrd_vol_qt = trdsize
			yld_pt = yield;
run;

data trace.trace_enhanced_final; set a6_trace_enhanced; run;
