/* Code for

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Appendix J of
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Semih Uslu: Pricing and Liquidity in Decentralized Asset Markets
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


This program summarizes the trade size heterogeneity using TRACE bond transactions data.	*/

/*	Erase the log and output.	*/
dm 'clear log'; dm 'clear output';

/*	Import the data	*/
/*	Bond transactions.	*/
data a1; set "/folders/myfolders/sasuser.v94/trace_enhanced_final.sas7bdat"; run; /* Enter your own directory information on this line. */
proc sort data = a1; by descending trdsize; run;

/*	Bond ratings.	*/
proc import out = agencies_ratings datafile="/folders/myfolders/sasuser.v94/Agencies_Ratings.xlsx" dbms=xlsx replace; run; /* Enter your own directory information on this line. */
proc import out = bond_rating_rdy datafile="/folders/myfolders/sasuser.v94/Bond_Rating.dta" dbms=dta replace; run; /* Enter your own directory information on this line. */
/*	Bring the corresponding numerical scores for FISD ratings.	*/
proc sql; create table b3 as select a.*, b.rating_score, b.rating_group, b.investment_grade from bond_rating_rdy a left join agencies_ratings b on a.rating = b.rating 
	and a.rating_type = b.rating_type; quit;
proc sort data = b3; by complete_cusip rating_type rating_date; run;
data b5r; set b3; where (rating_type = 'SPR' or rating_type = 'MR' or rating_type = 'FR') and rating_score ne .; run;
proc sort data = b5r; by complete_cusip rating_date descending rating_type; run;
proc sort data = b5r nodupkey; by complete_cusip rating_date; run;
data b6; set b5r; rating_year = year(rating_date); rating_month = month(rating_date); year_month_num = rating_year * 12 + rating_month; run;
proc sort data = b6; by complete_cusip descending rating_date; run;
proc sort data = b6 nodupkey; by complete_cusip descending year_month_num; run;

/*	Bring ratings to bond transactions.	*/
data a2 (keep = complete_cusip trdsize trans_date year_month_num); set a1; 
	trans_year = year(trans_date); 
	trans_month = month(trans_date); 
	year_month_num = trans_year * 12 + trans_month;
run;
proc sql; create table a3 as select a.*, b.rating_score from a2 a left join b6 b 
	on a.complete_cusip = b.complete_cusip and a.year_month_num >= b.year_month_num
	group by a.complete_cusip, a.year_month_num having b.year_month_num - a.year_month_num = max(b.year_month_num - a.year_month_num);
quit;

data a4; set a3; 
	investment_grade = 0; if rating_score <= 10 and rating_score ne . then investment_grade = 1; 
	a_above = 0; if rating_score <= 7 and rating_score ne . then a_above = 1;
	full_sample = 1;
	trans_year = year(trans_date);
run;
	
proc sql; create table a4_test as select trans_year, count(trans_date) as num_obs from a4 group by trans_year; quit;

data a5; set a4;
	transaction_group = 1;
		if trdsize > 1000 & trdsize <= 5000 then transaction_group = 2;
		if trdsize > 5000 & trdsize <= 25000 then transaction_group = 3;
		if trdsize > 25000 & trdsize <= 100000 then transaction_group = 4;
		if trdsize > 100000 & trdsize <= 500000 then transaction_group = 5;
		if trdsize > 500000 & trdsize <= 5000000 then transaction_group = 6;
		if trdsize > 5000000 then transaction_group = 7;
run;

data a6; set a5; where trans_year <= 2014; run;

data d1_2014; set a6; where trans_year = 2014; run;
data d1_2012_2014; set a6; where 2012 <= trans_year <= 2014; run;
data d1_2005_2014; set a5; where 2005 <= trans_year <= 2014; run;

/*	Summaries	*/
/*	2014	*/
proc summary data = d1_2014; by full_sample; var trdsize; output out=d1_2014_sum1 min= p1= p10= p50= p90= p99= max= mean= std= /autoname; run;
proc sort data = d1_2014; by investment_grade a_above; run;
proc summary data = d1_2014; by a_above; var trdsize; output out=d1_2014_sum2 min= p1= p10= p50= p90= p99= max= mean= std= /autoname; run;
proc summary data = d1_2014; by investment_grade; var trdsize; output out=d1_2014_sum3 min= p1= p10= p50= p90= p99= max= mean= std= /autoname; run;
data d1_2014_sum; set d1_2014_sum2 d1_2014_sum3 d1_2014_sum1; if a_above = 0 then delete; if investment_grade = 0 then delete; 
	std_normalized = trdsize_stddev / trdsize_mean; group = "2014_____"; run;
/*	2012 - 2014	*/
proc summary data = d1_2012_2014; by full_sample; var trdsize; output out=d1_2012_2014_sum1 min= p1= p10= p50= p90= p99= max= mean= std= /autoname; run;
proc sort data = d1_2012_2014; by investment_grade a_above; run;
proc summary data = d1_2012_2014; by a_above; var trdsize; output out=d1_2012_2014_sum2 min= p1= p10= p50= p90= p99= max= mean= std= /autoname; run;
proc summary data = d1_2012_2014; by investment_grade; var trdsize; output out=d1_2012_2014_sum3 min= p1= p10= p50= p90= p99= max= mean= std= /autoname; run;
data d1_2012_2014_sum; set d1_2012_2014_sum2 d1_2012_2014_sum3 d1_2012_2014_sum1; if a_above = 0 then delete; if investment_grade = 0 then delete; 
	std_normalized = trdsize_stddev / trdsize_mean; group = "2012-2014"; run;
/*	2005 - 2014	*/
proc summary data = d1_2005_2014; by full_sample; var trdsize; output out=d1_2005_2014_sum1 min= p1= p10= p50= p90= p99= max= mean= std= /autoname; run;
proc sort data = d1_2005_2014; by investment_grade a_above; run;
proc summary data = d1_2005_2014; by a_above; var trdsize; output out=d1_2005_2014_sum2 min= p1= p10= p50= p90= p99= max= mean= std= /autoname; run;
proc summary data = d1_2005_2014; by investment_grade; var trdsize; output out=d1_2005_2014_sum3 min= p1= p10= p50= p90= p99= max= mean= std= /autoname; run;
data d1_2005_2014_sum; set d1_2005_2014_sum2 d1_2005_2014_sum3 d1_2005_2014_sum1; if a_above = 0 then delete; if investment_grade = 0 then delete; 
	std_normalized = trdsize_stddev / trdsize_mean; group = "2005-2014"; run;

data d1_sum (drop = full_sample a_above investment_grade _type_); set d1_2014_sum d1_2012_2014_sum d1_2005_2014_sum; 
	group = "A_above";
		if investment_grade = 1 then group = "IG";
		if full_sample = 1 then group = "Full";
run;
proc export data = d1_sum dbms = xlsx outfile = "/folders/myfolders/sasuser.v94/d1_sum.xlsx" replace; run; /* Enter your own directory information on this line. */

/*	distributions	*/
proc sql; create table d2_2014_1 as select a_above, transaction_group, count(trdsize) as num_obs from d1_2014 group by a_above, transaction_group; quit;
proc sql; create table d2_2014_1b as select *, sum(num_obs) as total_obs from d2_2014_1 group by a_above; quit;
proc sql; create table d2_2014_2 as select investment_grade, transaction_group, count(trdsize) as num_obs from d1_2014 group by investment_grade, transaction_group; quit;
proc sql; create table d2_2014_2b as select *, sum(num_obs) as total_obs from d2_2014_2 group by investment_grade; quit;
proc sql; create table d2_2014_3 as select full_sample, transaction_group, count(trdsize) as num_obs from d1_2014 group by full_sample, transaction_group; quit;
proc sql; create table d2_2014_3b as select *, sum(num_obs) as total_obs from d2_2014_3 group by full_sample; quit;
data d3_2014; set d2_2014_1b d2_2014_2b d2_2014_3b; percentage = num_obs / total_obs; group = "2014_____"; run;

proc sql; create table d2_2012_2014_1 as select a_above, transaction_group, count(trdsize) as num_obs from d1_2012_2014 group by a_above, transaction_group; quit;
proc sql; create table d2_2012_2014_1b as select *, sum(num_obs) as total_obs from d2_2012_2014_1 group by a_above; quit;
proc sql; create table d2_2012_2014_2 as select investment_grade, transaction_group, count(trdsize) as num_obs from d1_2012_2014 group by investment_grade, transaction_group; quit;
proc sql; create table d2_2012_2014_2b as select *, sum(num_obs) as total_obs from d2_2012_2014_2 group by investment_grade; quit;
proc sql; create table d2_2012_2014_3 as select full_sample, transaction_group, count(trdsize) as num_obs from d1_2012_2014 group by full_sample, transaction_group; quit;
proc sql; create table d2_2012_2014_3b as select *, sum(num_obs) as total_obs from d2_2012_2014_3 group by full_sample; quit;
data d3_2012_2014; set d2_2012_2014_1b d2_2012_2014_2b d2_2012_2014_3b; percentage = num_obs / total_obs; group = "2012_2014"; run;

proc sql; create table d2_2005_2014_1 as select a_above, transaction_group, count(trdsize) as num_obs from d1_2005_2014 group by a_above, transaction_group; quit;
proc sql; create table d2_2005_2014_1b as select *, sum(num_obs) as total_obs from d2_2005_2014_1 group by a_above; quit;
proc sql; create table d2_2005_2014_2 as select investment_grade, transaction_group, count(trdsize) as num_obs from d1_2005_2014 group by investment_grade, transaction_group; quit;
proc sql; create table d2_2005_2014_2b as select *, sum(num_obs) as total_obs from d2_2005_2014_2 group by investment_grade; quit;
proc sql; create table d2_2005_2014_3 as select full_sample, transaction_group, count(trdsize) as num_obs from d1_2005_2014 group by full_sample, transaction_group; quit;
proc sql; create table d2_2005_2014_3b as select *, sum(num_obs) as total_obs from d2_2005_2014_3 group by full_sample; quit;
data d3_2005_2014; set d2_2005_2014_1b d2_2005_2014_2b d2_2005_2014_3b; percentage = num_obs / total_obs; group = "2005_2014"; run;

data d3_sum; set d3_2014 d3_2012_2014 d3_2005_2014; 
	if a_above = 0 then delete;
	if investment_grade = 0 then delete;
run;

proc export data = d3_sum dbms = dta outfile = "/folders/myfolders/sasuser.v94/d3_sum.dta" replace; run; /* Enter your own directory information on this line. */
